/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.konfiguration;

import java.util.Collections;
import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Es wird ein Strassenknoten modelliert.
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:49:24 $ / ($Author: Sans $)
 */
public class StrassenKnoten extends DavProxyObjekt 
{
  /**
   * Debug-Logger für Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  /**
   * Erstellt eine neue Instanz vom Typ <code>StrassenKnoten</code>
   *
   * @param davObjekt das zuzuordnente Dav-Objekt
   * @param typ Typ
   */
  public StrassenKnoten(SystemObject davObjekt, String typ)
  {
    super(davObjekt);
    _typ = typ;
  }

  /**
   * Länge in m dieses Straßensegments
   */
  private String _typ;

  /**
   * @return die Länge in m dieses Straßensegments
   */
  public String getTyp()
  {
    return _typ;
  }

  /**
   * Die von diesem Strassenknoten abgehenden äusseren Strassensegmente
   */
  private List<AeusseresStrassenSegment> _abgehendeAeussereSegmente
    = new Vector<AeusseresStrassenSegment>();

  /**
   * Read-Only-Zugriff auf die von diesem Strassenknoten abgehenden
   * äusseren Strassensegmente
   *
   * @return Liste der von diesem Strassenknoten abgehenden äusseren
   * Strassensegmente
   */
  public List<AeusseresStrassenSegment> getAbgehendeSegmente()
  {
    return Collections.unmodifiableList(_abgehendeAeussereSegmente);
  }

  /**
   * Fügt diesem Knoten ein abgehendes Segment hinzu
   *
   * @param segment das neue abgehende Segment
   */
  void addAbgehendesSegment(AeusseresStrassenSegment segment)
  {
    _abgehendeAeussereSegmente.add(segment);
  }

  /**
   * Die zu diesem Strassenknoten hinführenden äusseren Strassensegmente
   */
  private List<AeusseresStrassenSegment> _hinfuehrendeAeussereSegmente
    = new Vector<AeusseresStrassenSegment>();

  /**
   * Read-Only-Zugriff auf die zu diesem Strassenknoten hinführenden
   * äusseren Strassensegmente
   *
   * @return Liste der zu diesem Strassenknoten hinführenden äusseren
   * Strassensegmente
   */
  public List<AeusseresStrassenSegment> getHinfuehrendeSegmente()
  {
    return Collections.unmodifiableList(_hinfuehrendeAeussereSegmente);
  }

  /**
   * Fügt diesem Knoten ein hinführendes Segment hinzu
   *
   * @param segment das neue hinführende Segment
   */
  void addHinfuehrendesSegment(AeusseresStrassenSegment segment)
  {
    _hinfuehrendeAeussereSegmente.add(segment);
  }

  /**
   * Die zu diesem Strassenknoten gehörenden inneren Strassensegmente
   */
  private List<InneresStrassenSegment> _innereSegmente
    = new Vector<InneresStrassenSegment>();

  /**
   * Read-Only-Zugriff auf die zu diesem Strassenknoten gehörenden
   * inneren Strassensegmente
   *
   * @return Liste der zu diesem Strassenknoten gehörenden inneren
   * Strassensegmente
   */
  public List<InneresStrassenSegment> getInnereSegmente()
  {
    return Collections.unmodifiableList(_innereSegmente);
  }

  /**
   * Fügt diesem Knoten ein inneres Segment hinzu
   *
   * @param segment das neue hinführende Segment
   */
  void addInneresSegment(InneresStrassenSegment segment)
  {
    _innereSegmente.add(segment);
  }

  /**
   * Methode prüft ob es sich bei dem Knoten um ein Autobahnkreuz handelt
   *
   * @return Knoten ist Autobahnkreuz: true, sonst false
   */
  public boolean isAutobahnKreuz ()
  {
    return _typ.equals( "AutobahnKreuz" );
  }

  /**
   * Methode prüft ob es sich bei dem Knoten um ein Autobahndreieck handelt
   *
   * @return Knoten ist Autobahndreieck: true, sonst false
   */
  public boolean isAutobahnDreieck ()
  {
    return _typ.equals( "AutobahnDreieck" );
  }

  /**
   * Methode prüft ob es sich bei dem Knoten um ein Autobahnende handelt
   *
   * @return Knoten ist Autobahnende: true, sonst false
   */
  public boolean isAutobahnEnde ()
  {
    return _typ.equals( "AutobahnEnde" );
  }

  /**
   * Methode prüft ob es sich bei dem Knoten um einen Autobahnanfang handelt
   *
   * @return Knoten ist Autobahnanfang: true, sonst false
   */
  public boolean isAutobahnAnfang ()
  {
    return _typ.equals( "AutobahnAnfang" );
  }

  /**
   * Methode prüft ob es sich bei dem Knoten um eine Autobahnanschlussstelle handelt
   *
   * @return Knoten ist eine Autobahnanschlussstelle: true, sonst false
   */
  public boolean isAutobahnAnschlussStelle ()
  {
    return _typ.equals( "AutobahnAnschlussStelle" );
  }

  /**
   * Methode prüft ob es sich bei dem Knoten um einen sonstigen Knoten handelt
   * 
   * @return Knoten ist ein sonstiger Knoten: true, sonst false
   */
  public boolean isSonstigerKnoten ()
  {
    return _typ.equals( "SonstigerKnoten" );
  }
}
