/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.rdstmc;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Applikationsklasse zum Löschen von Simulationsstrecken
 *
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2009/01/27 13:54:22 $ / ($Author: Sans $)
 */
public class LoescheRDSMeldungen implements Runnable, StandardApplication
{
  /**
   * Debug-Logger für Logging-Ausgaben
   */
  private Debug _debug;

  /**
   * -pidPrefix von Kommandozeile
   */
  private String _pidPrefix;

  /**
   * Datenverteiler-Verbindung
   */
  private ClientDavInterface _connection;

  /**
   * Mengenbezeichner für die dynamische Menge
   */
  static final String MENGE = "RDSMeldungen";

  /**
   * Default- Konstruktor.<br>
   * Wird typischerweise von <code>main</code> aufgerufen
   */
  public LoescheRDSMeldungen()
  {
    try
    {
      Thread.sleep(100);
    }
    catch (InterruptedException e)
    {
    }
  }

  /**
   * Überschriebene Methode von <code>StandardApplication</code>,
   * die die speziellen Startparameter auswertet
   *
   * @param argumentList
   *  siehe sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList)
  {
    _debug = Debug.getLogger();

    _pidPrefix = argumentList.fetchArgument("-pidPrefix=").asString();

    argumentList.fetchUnusedArguments();
  }

  /**
   * Überschriebene Methode von StandardApplication, die die Initialisierung durchführt
   *
   * @param connection
   * siehe sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection)
  {
    _connection = connection;

    new Thread(this).start();
  }

  public void run()
  {
    SystemObjectType objTyp = _connection.getDataModel().getType("typ.rdsMeldung");
    SystemObjectType confTyp = _connection.getDataModel().getType("typ.landesMeldeStelle");
    int anzahl = 0;

    _debug.info("Alle Objekte vom Typ " + objTyp.getPid() + ", deren PID mit \"" + _pidPrefix +
        "\" beginnt\n- werden gelöscht\n- entsprechende dynamische Mengen werden bereinigt");

    for (SystemObject obj: objTyp.getElementsInPeriod(1, (new Date()).getTime()))
    {
      if (obj.isValid() && obj.getPid().startsWith(_pidPrefix))
      {
        boolean fehler = false;
        Map<SystemObject, MutableSet> bereinigteMengen = new HashMap<SystemObject, MutableSet>();

        for (SystemObject conf: confTyp.getObjects())
        {
          if (conf.isValid())
          {
            MutableSet menge = ((ConfigurationObject)conf).getMutableSet(MENGE);
            if (menge != null)
            {
              try
              {
                menge.remove(obj);
                bereinigteMengen.put(conf, menge);
                _debug.info("Objekt \"" + obj.getPid() + "\" bereinigt in Menge \"" + MENGE + "\" von Konfigurationsobjekt " + conf.getName());
              }
              catch(ConfigurationChangeException e)
              {
                fehler = true;
                _debug.error("Objekt \"" + obj.getPid() + "\" konnte nicht bereingt werden in Menge \"" + MENGE + "\" von Konfigurationsobjekt " + conf.getName(), e);
                break;
              }
            }
          }
        }
        if (!fehler)
        {
          try
          {
            obj.invalidate();
            anzahl += 1;
          }
          catch(ConfigurationChangeException e)
          {
            fehler = true;
            _debug.error("Objekt \"" + obj.getPid() + "\" konnte nicht gelöscht werden", e);
          }
        }
        if (fehler)
        {
          for (SystemObject conf: bereinigteMengen.keySet())
          {
            MutableSet menge = bereinigteMengen.get(conf);
            try
            {
              menge.add(obj);
            }
            catch(ConfigurationChangeException e)
            {
              _debug.error("Bereinigte Menge \"" + MENGE + "\" des Konfigurationsobjekts " + conf.getName() + "\" konnte nicht wiederhergestellt werden", e);
              continue;
            }
          }
          continue;
        }
      }
    }
    _debug.info(anzahl + " Objekt(e) vom Typ " + objTyp.getPid() + " wurden erfolgreich gelöscht");
    System.exit(0);
  }
  
  /**
   * Programmeinstieg
   * @param arguments Parameter.
   */
  public static void main(String[] arguments)
  {
    StandardApplicationRunner.run(new LoescheRDSMeldungen(), arguments);
  }
  
}
